#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jul  9 20:45:33 2020

@author: lucia
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import ode
from Hodgkin_Huxley_EDOs import Hodgkin_Huxley

'''
===============================================================================
Modelo de Hodgkin-Huxley
Hodgkin-Huxley model of excitable barnacle muscle fiber
reviewed in Rinzel (1990) Bulletin of Mathematical Biology 52 pp. 5-23.
===============================================================================
'''

# Asignacion de constantes fisicas---------------------------------------------
E_N = 55
E_K = -72
g_N_bar = 120.0
g_K_bar = 36.0
g_leak = 0.30
E_leak = -49.0
C_M = 1.0

# Obtencion del potencial de accion--------------------------------------------
# Crear una instancia 'ode' para resolver el sistema de ecuaciones
# diferenciales definidas en Hodgkin_Huxley(), y establecer el método de
# resolución a 'dopri5' (Runge Kutta de orden (4,5)).
solver = ode(Hodgkin_Huxley)
solver.set_integrator('dopri5')

# Pasar los valores de los parámetros al resolvedor. Éstos se pasan a la función 
# 'Hodgkin_Huxley' cuando el resolvedor la llama 
solver.set_f_params(E_N, E_K, g_N_bar, g_K_bar, g_leak, E_leak, C_M)

# Condiciones Iniciales [V m h n]
Vm0 = -65 #mV
m0 = 0.02 
h0 = 0.72
n0 = 0.26

# Establecer los valores iniciales (t0 y S0)
t0 = 0.0
S0 = [Vm0, m0, h0, n0]
solver.set_initial_value(S0, t0)

# Intervalo de simulacion
tf = 150  #ms
N = 450
t = np.linspace(t0, tf, N)

# Crear un array donde almacenar las salidas.
# Incluir los valores iniciales en el arreglo.
S = np.empty((N, len(S0)))
S[0] = S0

# Llamar iterativamente el metodo 'integrate' para actualizar la solución hasta
#  el tiempo t[k] y guardarla en S[k].
k = 1
while solver.successful() and solver.t < tf:
    solver.integrate(t[k])
    S[k] = solver.y
    k += 1

# Graficar las salidas
plt.figure()
plt.plot(t,S[:,0],'-');
plt.xlabel('Tiempo (ms)');
plt.ylabel('Voltaje (mV)');
plt.title('Voltaje de membrana');

plt.figure()
plt.plot(t,S[:,1],'-', label='m')
plt.plot(t,S[:,2],'-', label='h')
plt.plot(t,S[:,3],'-', label='n')
plt.legend()
plt.xlabel('Tiempo (ms)')
plt.title('Apertura de canales')
